Type: Package
Package: MetaScope
Title: Tools and functions for preprocessing 16S and metagenomic
        sequencing microbiome data
Version: 1.99.10
Authors@R: c(
    person("Sean", "Lu", , "seanlu96@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0009-0007-8005-6125")),
    person("Aubrey", "Odom", , "aodom@bu.edu", role = c("aut"),
           comment = c(ORCID = "0000-0001-7113-7598")),
    person("Rahul", "Varki", , "rvarki@bu.edu", role = "aut",
           comment = c(ORCID = "0009-0003-5721-9484")),
    person("W. Evan", "Johnson", , "wej@bu.edu", role = "aut",
           comment = c(ORCID = "0000-0002-6247-6595"))
  )
Description: This package contains tools and methods for preprocessing
    microbiome data. Functionality includes library generation,
    demultiplexing, alignment, and microbe identification.  It is in part
    an R translation of the PathoScope 2.0 pipeline.
License: GPL (>= 3)
URL: https://github.com/wejlab/metascope
        https://wejlab.github.io/metascope-docs/
BugReports: https://github.com/wejlab/MetaScope/issues
Depends: R (>= 4.2.0)
Imports: BiocFileCache, Biostrings, data.table (>= 1.16.2), dplyr,
        ggplot2, magrittr, Matrix, MultiAssayExperiment, purrr,
        Rbowtie2, readr, rlang, Rsamtools, S4Vectors, stringr,
        SummarizedExperiment, taxonomizr, tibble, tidyr, tools
Suggests: animalcules, BiocStyle, biomformat, GenomicRanges, IRanges,
        knitr, lintr, plyr, R.utils, RCurl, rmarkdown, Rsubread,
        spelling, sys, testthat, usethis
Enhances: BiocParallel
VignetteBuilder: knitr
BiocType: Software
biocViews: MicrobiomeData, ReproducibleResearch, SequencingData
Encoding: UTF-8
Language: en-US
LazyData: FALSE
RoxygenNote: 7.3.3
git_url: https://git.bioconductor.org/packages/MetaScope
git_branch: devel
git_last_commit: d49e047
git_last_commit_date: 2025-11-07
Repository: Bioconductor 3.23
Date/Publication: 2025-11-11
NeedsCompilation: no
Packaged: 2025-11-11 16:38:56 UTC; biocbuild
Author: Sean Lu [aut, cre] (ORCID: <https://orcid.org/0009-0007-8005-6125>),
  Aubrey Odom [aut] (ORCID: <https://orcid.org/0000-0001-7113-7598>),
  Rahul Varki [aut] (ORCID: <https://orcid.org/0009-0003-5721-9484>),
  W. Evan Johnson [aut] (ORCID: <https://orcid.org/0000-0002-6247-6595>)
Maintainer: Sean Lu <seanlu96@gmail.com>
