# Generated by roxygen2: do not edit by hand

export(align_target)
export(align_target_bowtie)
export(blast_reassignment)
export(convert_animalcules)
export(convert_animalcules_patho)
export(convert_animalcules_silva)
export(count_matches)
export(download_accessions)
export(download_refseq)
export(download_refseq_16S)
export(extract_reads)
export(filter_host)
export(filter_host_bowtie)
export(get_children)
export(meta_demultiplex)
export(metascope_blast)
export(metascope_id)
export(metascope_inspect)
export(metascope_inspect_usage)
export(mk_bowtie_index)
export(mk_subread_index)
export(run_metascope)
importFrom(magrittr,"%>%")
importFrom(rlang,.data)
