% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metascope_blast.R
\name{add_in_taxa}
\alias{add_in_taxa}
\title{Adds in taxa if silva database
Returns MetaScope Table with silva taxa in separate columns}
\usage{
add_in_taxa(metascope_id_in, caching, path_to_write)
}
\arguments{
\item{metascope_id_in}{MetaScope ID file with silva taxa}

\item{caching}{Boolean for if all_silva_headers.rds is already downloaded}

\item{path_to_write}{Path to save all_silva_headers.rds}
}
\value{
Data.frame of taxonomy information
}
\description{
Adds in taxa if silva database
Returns MetaScope Table with silva taxa in separate columns
}
\keyword{internal}
